using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;
using WorldServerAgent;
using System.Threading;
using System.Globalization;

namespace DemoClarionOscar
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		double prad = 0;
		private Utils utils;
		private string aux="";
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.3");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			
			utils = new Utils();			
			
			try
            {
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
                    worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
					
					agent.Oss.Name = creatureId;
					agent.Oss.LongName = creatureName;
					
					Console.Out.WriteLine("Creature created ID: " + agent.Oss.Name + "Name: " + agent.Oss.LongName);
					
					// Creating the world elements
					if(!utils.StartWorld(worldServer))
						return;
					
                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
						
						// Getting the leaflet
						string leaflet = worldServer.SendLeaflet();						
						utils.SetLeaflet(agent.Oss, leaflet);
						
                    }

                    //Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{   
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation()
        {
			IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {
                response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;

            }
						
			if(utils.UpdateCreature(agent.Oss, response))
			{
				Console.Out.WriteLine("GOALS COMPLETED !!!");
				Console.Out.WriteLine("--------------------------");
				Console.Out.WriteLine("THE GAME IS OVER !!!");
				Console.Out.WriteLine("--------------------------");
				worldServer.SendSetGoTo(creatureId,0,0, agent.Oss.X1 ,agent.Oss.Y1); // Stop the creature
				agent.Abort(false);
			}
			
			//Console.Out.WriteLine("Objects in the knapsack " + agent.Oss.knapsack.Count.ToString());
			
            return response;
        }

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {
			Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
			
            if (worldServer != null && worldServer.IsConnected)
            {
                switch (externalAction)
                {
                    case ClarionAgentActionType.DO_NOTHING:
                        // Do nothing as the own value says
                        break;
					
                    case ClarionAgentActionType.ROTATE_CLOCKWISE:
					    double newprad = prad + Math.PI/1000;
					    while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        worldServer.SendSetAngle(creatureId, 0, 0, newprad);
                        break;
					
                    case ClarionAgentActionType.GO_AHEAD:
                        worldServer.SendSetAngle(creatureId, 2, 2, prad);
                        break;
					
					case ClarionAgentActionType.GO_TO_JEWEL:						
						worldServer.SendSetGoTo(creatureId,2,2,agent.param[0] ,(double)agent.param[1]);
						break;
					
					case ClarionAgentActionType.GO_TO_FOOD:
						worldServer.SendSetGoTo(creatureId,2,2,agent.param_food[0] ,agent.param_food[1]);
						
						if((double)agent.param_food[2] < 25)							
							worldServer.SendEatIt(creatureId, agent.foodName);
					
						break;
					
					case ClarionAgentActionType.SACK_IT:
						string salida = worldServer.SendSackIt(creatureId, agent.SackItItem.Name);
					
						if(salida.Length > 0)					
						{
							if(!salida.Equals(aux))
							{
								//Console.Out.WriteLine(salida);
								agent.SackItItem.Name = agent.SackItItem.Name.Trim();
								agent.Oss.knapsack.Add(agent.SackItItem);
								aux = salida;								
							}						
						}
						
					break;
					
                    default:
                        break;
                }
            }
        }

        #endregion
	}
}

