using System;
using System.Text;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;
using System.Collections.Generic;

namespace DemoClarionOscar
{
	public class Utils
	{
		#region Public Atributes
		
		/// <summary>
		/// This is the list of all the goals
		/// </summary>
		public IList<Goal> GoalsList = new List<Goal>();
		
		/// <summary>
		/// Colors type.
		/// </summary>
		public enum ColorsType
		{
			Red,
			Green, 
			Blue,
			Yellow,
			Magenta,
			White
		}
		
		public string[] ColorList = {"Red","Green", "Blue", "Yellow", "Magenta", "White"};
		
		#endregion
		
		#region Public Methods
				
		/// <summary>
        /// Creates the elements in the world, i.e. the food, jewels etc
        /// </summary>        
        /// <returns> true if everything is ok, false otherwise </returns>
		public bool StartWorld(WorldServer world)
		{
			try
            {
                if (world != null && world.IsConnected)
                {   
					// Building externals bricks
					world.NewBrick(4, 0, 0, 10, 600);
					world.NewBrick(4, 0, 0, 800, 10);
					world.NewBrick(4, 790, 0, 800, 600);
					world.NewBrick(4, 0, 590, 800, 600);
															
					Random r = new Random(DateTime.Now.Millisecond);
					int pos_x = 0;
					int pos_y = 0;
					
					// Creating 5 jewels of each color randomly
					for(int j = 0; j < 6; j++)
					{					
						for(int i = 0; i < 5; i++)
						{
							pos_x = (int)r.Next(70, 740);
							pos_y = (int)r.Next(70, 540);
							
							world.NewJewel(j, pos_x, pos_y);
						}
					}
										
					// Creating 3 food randomly
					for(int i = 0; i < 3; i++)
					{
						pos_x = (int)r.Next(70, 740);
						pos_y = (int)r.Next(70, 540);
						
						world.NewFood(0, pos_x, pos_y);
					}
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
				return false;
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
				return false;
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
				return false;
            }
			
			return true;
		}
						
		/// <summary>
        /// Set the Leaflet list
        /// </summary>        
        /// <returns> true if everything was fine, false otherwise </returns>
		public Boolean SetLeaflet(Creature oss, string leaflet = null)
		{						
			int aux = 0;
						
			if (!String.IsNullOrWhiteSpace(leaflet))
            {
                string[] tokens = leaflet.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
				
				for(int i = 0; i < tokens.Length; i++)
				{
					if(tokens[i].Equals("Red") ||  tokens[i].Equals("Blue") || tokens[i].Equals("Green") || tokens[i].Equals("Magenta") || tokens[i].Equals("White") || tokens[i].Equals("Yellow"))
					{
						Goal goal = new Goal();						
						goal.Color = tokens[i];						
						goal.NumRequest = Convert.ToInt32(tokens[i+1]);						
						GoalsList.Add(goal);						
					}					
				}
            }			
			Console.Out.WriteLine("\n The leaflet is: " + leaflet);						
			return true;
		}
		
		/// <summary>
		/// Updates the creature information.
		/// </summary>
		/// <param name='oss'>
		/// Creature in the client side
		/// </param>
		/// <param name='objetos'>
		/// List of objects seen by the creature, including itself
		/// </param>
		public bool UpdateCreature(Creature oss, IList<Thing> objetos)
		{
			Creature aux = (Creature)objetos[0];
			
			oss.CategoryId = aux.CategoryId;						
			oss.Fuel = aux.Fuel;
			oss.HasLeaflet = aux.HasLeaflet;
			oss.index = aux.index;
			oss.IsOccluded = aux.IsOccluded;
			oss.LeafletList = aux.LeafletList;
			oss.Material = aux.Material;
			oss.MotorSystem = aux.MotorSystem;
			oss.Name = aux.Name;
			oss.NumberOfLeaflets = aux.NumberOfLeaflets;
			oss.Pitch = aux.Pitch;
			oss.Reward = aux.Reward;
			oss.Size = aux.Size;
			oss.Speed = aux.Speed;
			oss.Wheel = aux.Wheel;
			oss.X1 = aux.X1;
			oss.X2 = aux.X2;
			oss.Y1 = aux.Y1;
			oss.Y2 = aux.Y2;
			
			foreach(Thing ithing in oss.knapsack)
			{
				foreach (Leaflet ileaf in oss.LeafletList)
				{
					foreach(Goal igoal in ileaf.GoalsList)
					{
						if(!igoal.completed)
						{	
							if(igoal.Color.Trim().Equals(ithing.Material.Color.Trim()))
							{
								igoal.NumStack =  igoal.NumStack + 1;
								
								if(igoal.NumStack == igoal.NumRequest)
								{
									igoal.completed = true;
								}
								
								goto EndOfLoop;
							}
						}
					}
				}
				EndOfLoop: ;
			}
			
			
			// Check if the creature finished the goals
			
			bool Finish = true;
			
			foreach (Leaflet ileaf in oss.LeafletList)
			{
				foreach(Goal igoal in ileaf.GoalsList)
				{
					if(!igoal.completed)
					{
						Finish = false;
						
						goto EndOfLoop2;
					}
				}
			}
			
			EndOfLoop2: ;
			
			return Finish;			
		}		
		#endregion
	}
}

